package sun.awt.image;

import java.lang.ref.WeakReference;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.awt.image.BufferStrategy;

public abstract class VSyncedBSManager
{
    private static VSyncedBSManager theInstance;
    private static final boolean vSyncLimit;
    
    public VSyncedBSManager() {
    }
    
    private static VSyncedBSManager getInstance(boolean b) {
        if (VSyncedBSManager.theInstance == null && b) {
            VSyncedBSManager.theInstance = (VSyncedBSManager.vSyncLimit ? new VSyncedBSManager.SingleVSyncedBSMgr() : new VSyncedBSManager.NoLimitVSyncBSMgr());
        }
        return VSyncedBSManager.theInstance;
    }
    
    abstract boolean checkAllowed(BufferStrategy p0);
    
    abstract void relinquishVsync(BufferStrategy p0);
    
    public static boolean vsyncAllowed(BufferStrategy bufferStrategy) {
        return getInstance(true).checkAllowed(bufferStrategy);
    }
    
    public static synchronized void releaseVsync(BufferStrategy bufferStrategy) {
        VSyncedBSManager instance = getInstance(false);
        if (instance != null) {
            instance.relinquishVsync(bufferStrategy);
        }
    }
    
    static {
        vSyncLimit = (boolean)Boolean.valueOf((String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.vsynclimit", "true")));
    }
    
    private static final class NoLimitVSyncBSMgr extends VSyncedBSManager
    {
        @Override
        boolean checkAllowed(BufferStrategy bufferStrategy) {
            return true;
        }
        
        @Override
        void relinquishVsync(BufferStrategy bufferStrategy) {
        }
    }
    
    private static final class SingleVSyncedBSMgr extends VSyncedBSManager
    {
        private WeakReference<BufferStrategy> strategy;
        
        public synchronized boolean checkAllowed(BufferStrategy referent) {
            if (this.strategy != null) {
                BufferStrategy bufferStrategy = (BufferStrategy)this.strategy.get();
                if (bufferStrategy != null) {
                    return bufferStrategy == referent;
                }
            }
            this.strategy = new WeakReference(referent);
            return true;
        }
        
        public synchronized void relinquishVsync(BufferStrategy bufferStrategy) {
            if (this.strategy != null && (BufferStrategy)this.strategy.get() == bufferStrategy) {
                this.strategy.clear();
                this.strategy = null;
            }
        }
    }
}
